import sys
sys.path.append("../src")

import datetime

import toolbox


def assert_projects_same(backend1, backend2):
	projects1 = list(backend1.get_projects())
	projects1.sort()

	projects2 = list(backend2.get_projects())
	projects2.sort()

	assert projects1 == projects2


def assert_locations_same(backend1, backend2):
	locations1 = list(backend1.get_locations())
	locations1.sort()

	locations2 = list(backend2.get_locations())
	locations2.sort()

	assert locations1 == locations2


def assert_tasks_same(backend1, backend2, projId):
	tasks1 = list(backend1.get_tasks_with_details(projId))
	tasks1.sort()

	tasks2 = list(backend2.get_tasks_with_details(projId))
	tasks2.sort()

	assert tasks1 == tasks2


def exercise_projects(backend):
	PROJECT_1_NAME = "p1"
	PROJECT_1_NEW_NAME = "p1_1"
	PROJECT_2_NAME = "p2"

	projects = list(backend.get_projects())
	assert len(projects) == 0

	# Verify add/getters
	proj1Id = backend.add_project(PROJECT_1_NAME)

	proj = backend.get_project(proj1Id)
	assert proj["name"] == PROJECT_1_NAME
	assert proj["id"] == proj1Id
	assert proj["isVisible"]
	assert not proj["isMeta"]

	projects = list(backend.get_projects())
	assert len(projects) == 1
	assert proj == projects[0]

	lookupProj = backend.lookup_project(PROJECT_1_NAME)
	assert proj == lookupProj

	# verify name/getters
	backend.set_project_name(proj1Id, PROJECT_1_NEW_NAME)

	proj = backend.get_project(proj1Id)
	assert proj["name"] == PROJECT_1_NEW_NAME
	assert proj["id"] == proj1Id
	assert proj["isVisible"]
	assert not proj["isMeta"]

	projects = list(backend.get_projects())
	assert len(projects) == 1
	assert proj == projects[0]

	lookupProj = backend.lookup_project(PROJECT_1_NEW_NAME)
	assert proj == lookupProj

	# verify visibility/getters
	backend.set_project_visibility(proj1Id, False)

	proj = backend.get_project(proj1Id)
	assert proj["name"] == PROJECT_1_NEW_NAME
	assert proj["id"] == proj1Id
	assert not proj["isVisible"]
	assert not proj["isMeta"]

	projects = list(backend.get_projects())
	assert len(projects) == 1
	assert proj == projects[0]

	lookupProj = backend.lookup_project(PROJECT_1_NEW_NAME)
	assert proj == lookupProj

	# verify second project while modifying first
	proj2Id = backend.add_project(PROJECT_2_NAME)

	proj = backend.get_project(proj1Id)
	assert proj["name"] == PROJECT_1_NEW_NAME
	assert proj["id"] == proj1Id
	assert not proj["isVisible"]
	assert not proj["isMeta"]

	lookupProj = backend.lookup_project(PROJECT_1_NEW_NAME)
	assert proj == lookupProj

	proj = backend.get_project(proj2Id)
	assert proj["name"] == PROJECT_2_NAME
	assert proj["id"] == proj2Id
	assert proj["isVisible"]
	assert not proj["isMeta"]

	lookupProj = backend.lookup_project(PROJECT_2_NAME)
	assert proj == lookupProj

	backend.set_project_name(proj1Id, PROJECT_1_NAME)

	proj = backend.get_project(proj1Id)
	assert proj["name"] == PROJECT_1_NAME
	assert proj["id"] == proj1Id
	assert not proj["isVisible"]
	assert not proj["isMeta"]

	lookupProj = backend.lookup_project(PROJECT_1_NAME)
	assert proj == lookupProj

	proj = backend.get_project(proj2Id)
	assert proj["name"] == PROJECT_2_NAME
	assert proj["id"] == proj2Id
	assert proj["isVisible"]
	assert not proj["isMeta"]

	lookupProj = backend.lookup_project(PROJECT_2_NAME)
	assert proj == lookupProj

	# verify visibility/getters
	backend.set_project_visibility(proj1Id, True)

	proj = backend.get_project(proj1Id)
	assert proj["name"] == PROJECT_1_NAME
	assert proj["id"] == proj1Id
	assert proj["isVisible"]
	assert not proj["isMeta"]

	lookupProj = backend.lookup_project(PROJECT_1_NAME)
	assert proj == lookupProj

	proj = backend.get_project(proj2Id)
	assert proj["name"] == PROJECT_2_NAME
	assert proj["id"] == proj2Id
	assert proj["isVisible"]
	assert not proj["isMeta"]

	lookupProj = backend.lookup_project(PROJECT_2_NAME)
	assert proj == lookupProj


def exercise_locations(backend):
	locations = list(backend.get_locations())
	assert len(locations) == 0


def exercise_task(backend, proj1Id, proj2Id):
	TASK_1_NAME = "t1"
	TASK_1_NEW_NAME = "t1_1"
	TASK_1_DUEDATE = toolbox.Optional(datetime.datetime.now())
	TASK_2_NAME = "t1"

	proj1Tasks = list(backend.get_tasks_with_details(proj1Id))
	assert len(proj1Tasks) == 0
	proj2Tasks = list(backend.get_tasks_with_details(proj2Id))
	assert len(proj2Tasks) == 0

	taskId = backend.add_task(proj1Id, TASK_1_NAME)
	task = backend.get_task_details(taskId)
	assert task["name"] == TASK_1_NAME
	assert task["projId"] == proj1Id
	assert task["id"] == taskId

	proj1Tasks = list(backend.get_tasks_with_details(proj1Id))
	assert len(proj1Tasks) == 1
	proj2Tasks = list(backend.get_tasks_with_details(proj2Id))
	assert len(proj2Tasks) == 0

	backend.set_name(taskId, TASK_1_NEW_NAME)
	task = backend.get_task_details(taskId)
	assert task["name"] == TASK_1_NEW_NAME

	assert task["dueDate"] == toolbox.Optional(), "%r not optional" % task["dueDate"]
	backend.set_duedate(taskId, TASK_1_DUEDATE)
	task = backend.get_task_details(taskId)
	assert task["dueDate"] == TASK_1_DUEDATE
